#ifndef _CEGOHAVINGCOND_H_INCLUDED_
#define _CEGOHAVINGCOND_H_INCLUDED_
///////////////////////////////////////////////////////////////////////////////
//                                                         
// CegoHavingCond.h  
// ----------------
// Cego having condition
//                                                        
// Design and Implementation by Bjoern Lemke
//
// (C)opyright 2000-2025 Bjoern Lemke
//
// INTERFACE MODULE
//
// Class: CegoHavingCond
// 
// Description: Group having condition
//
// Status: CLEAN
//
///////////////////////////////////////////////////////////////////////////////

// CEGO INCLUDES

#include "CegoHavingDesc.h"

class CegoHavingCond  {
    
public:
    
    enum ConditionMode { AND, OR, HAVINGDESC };
    
    CegoHavingCond(char* buf, CegoDistManager *pGTM, int tabSetId);    
    CegoHavingCond(CegoHavingCond* pCond, CegoHavingDesc *pDesc, ConditionMode mode);
    CegoHavingCond(CegoHavingDesc *pDesc);
    ~CegoHavingCond();

    ConditionMode getMode() { return _mode; }
    CegoHavingCond* getCond() { return _pCond; }
    CegoHavingDesc* getDesc() { return _pDesc; }    
    void encode(char *buf);
    void decode(char *buf, CegoDistManager *pGTM, int tabSetId);
    int getEncodingLength() const;
    Chain getId() const;
    Chain toChain(int defTabSetId, const Chain& indent = Chain("")) const;
    Chain dbFormat(CegoDatabaseFormater* pForm) const;
    CegoHavingCond* clone(bool isAttrRef = false);
    ListT<CegoAttrDesc*> getAttrRefList() const;
    void setFieldListArray(ListT<CegoField>* pFL);
    void setAggregationValue(ListT<CegoField>& jfl);
    bool match() const;
    ListT<CegoAggregation*> getAggregationList();
    ListT<CegoExpr*> getExprList();

    CegoHavingCond& operator = (const CegoHavingCond& c);    
    friend ostream& operator << (ostream& s, const CegoHavingCond& c);
    
private:

    ConditionMode _mode;
    CegoHavingCond* _pCond;
    CegoHavingDesc* _pDesc;    
};
#endif
