#ifndef _CEGOHAVINGDESC_H_INCLUDED_
#define _CEGOHAVINGDESC_H_INCLUDED_
///////////////////////////////////////////////////////////////////////////////
//                                                         
// CegoHavingDesc.h
// ----------------
// Cego having clause structure class definition
//          
// Design and Implementation by Bjoern Lemke
//
// (C)opyright 2000-2025 Bjoern Lemke
//
// INTERFACE MODULE
//
// Class: CegoHavingDesc
// 
// Description: The CegoHavingDesc class is a container class for the query having clause representation
//
// Status: CLEAN
//
///////////////////////////////////////////////////////////////////////////////

// LFC INCLUDES
#include <lfcbase/Chain.h>
#include <lfcbase/ListT.h>

// CEGO INCLUDES
#include "CegoDistManager.h"
#include "CegoComparison.h"
#include "CegoExpr.h"
#include "CegoField.h"
#include "CegoAggregation.h"

class CegoDatabaseFormater;
class CegoHavingCond;

class CegoHavingDesc {
    
public:
    
    CegoHavingDesc(char* buf, CegoDistManager *pGTM, int tabSetId);    
    CegoHavingDesc(CegoExpr* pAggExpr, CegoExpr* pExpr, const CegoComparison& comp);
    CegoHavingDesc(CegoHavingCond* pCond);
    ~CegoHavingDesc();

    CegoExpr* getAggExpr();
    CegoExpr* getExpr();
    CegoHavingCond* getCond();
    CegoComparison getComparison();
    ListT<CegoAttrDesc*> getAttrRefList() const;
    void encode(char *buf);
    void decode(char *buf, CegoDistManager *pGTM, int tabSetId);
    int getEncodingLength() const;
    Chain getId() const;
    Chain toChain(int defTabSetId, const Chain& indent = Chain("")) const;
    Chain dbFormat(CegoDatabaseFormater* pForm) const;
    CegoHavingDesc* clone(bool isAttrRef = false);
    void setFieldListArray(ListT<CegoField>* pFL);
    void setAggregationValue(ListT<CegoField>& jfl);
    bool match() const;
    ListT<CegoAggregation*> getAggregationList();
    ListT<CegoExpr*> getExprList();

    friend ostream& operator << (ostream& s, const CegoHavingDesc& p);
    
private:

    void setAggregationValueForExpr(CegoExpr* pExpr, ListT<CegoField>& jfl);
    bool matchFieldValue(const CegoFieldValue& fv1, const CegoFieldValue& fv2, CegoComparison comp) const;
    	
    CegoExpr* _pAggExpr;
    CegoExpr* _pExpr;
    CegoComparison _comp;
    CegoHavingCond* _pCond;
};
#endif
